class OrderTakerException(Exception):
    def __init__(self, code, description):
        super(OrderTakerException, self).__init__()
        self._code = int(code)
        self._description = description

    @property
    def error_code(self):
        return self._code

    @property
    def error_description(self):
        return self._description

    def __str__(self):
        return "Error code: {0}; Error description: {1}".format(self._code, self._description)
