from datetime import time


class PicklistCleanerManagerConfig(object):
    def __init__(
        self,                   # type: PicklistCleanerManagerConfig
        start_time,             # type: time
        end_time,               # type: time
        days_to_keep_data,      # type: int
    ):
        # type: (...) -> None

        self.start_time = start_time
        self.end_time = end_time
        self.days_to_keep_data = days_to_keep_data
