from ._TranslatedMessage import TranslatedMessage


class TotaledMessage(object):
    def __init__(
        self,                       # type: TotaledMessage
        totalization_message,       # type: TranslatedMessage
    ):
        # type: (...) -> None

        self.totalization_message = totalization_message
